//******************************************************************************
//
// Copyright (c) 2016 Intel Corporation. All rights reserved.
// Copyright (c) 2015 Microsoft Corporation. All rights reserved.
//
// This code is licensed under the MIT License (MIT).
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//
//******************************************************************************

#pragma once

#import "UIColorInternal.h"

class CGVectorImageBacking;

class CGVectorImage : public __CGImage {
public:
    CGVectorImage(DWORD width, DWORD height, __CGSurfaceFormat fmt);
    CGImageRef Rasterize(CGRect* insets);

    inline CGVectorImageBacking* VectorBacking() {
        return (CGVectorImageBacking*)Backing();
    }
};

enum VectorDrawingCommandType { vectorDrawingImage, vectorDrawingLine };

class CGVectorDrawingCommand {
public:
    VectorDrawingCommandType _type;
    CGImageRef _image;
    CGRect _src, _dest;
    CGPoint _start, _end;
    float _lineWidth;
    __CGColorQuad _color;

    CGVectorDrawingCommand(CGImageRef image, CGRect src, CGRect dst);
    CGVectorDrawingCommand(CGPoint lineStart, CGPoint lineEnd, __CGColorQuad color, float lineWidth);
    ~CGVectorDrawingCommand();
};

class CGVectorImageBacking : public CGImageBacking {
private:
    int _width, _height;
    __CGSurfaceFormat _surfaceFmt;
    CGColorSpaceModel _colorSpaceModel;
    CGVectorDrawingCommand* _drawCommand;

public:
    CGVectorImageBacking(DWORD width, DWORD height, __CGSurfaceFormat fmt);

    ~CGVectorImageBacking();

    CGImageRef Copy();

    CGContextImpl* CreateDrawingContext(CGContextRef base);
    void GetPixel(int x, int y, float& r, float& g, float& b, float& a);
    int InternalWidth();
    int InternalHeight();
    int Width();
    int Height();
    int BytesPerRow();
    int BytesPerPixel();
    int BitsPerComponent();
    void GetSurfaceInfoWithoutPixelPtr(__CGSurfaceInfo* surfaceInfo);
    __CGSurfaceFormat SurfaceFormat();
    CGColorSpaceModel ColorSpaceModel();
    CGBitmapInfo BitmapInfo();
    void* StaticImageData();
    void* LockImageData();
    void ReleaseImageData();
    cairo_surface_t* LockCairoSurface();
    void ReleaseCairoSurface();
    void SetFreeWhenDone(bool freeWhenDone);

    void DrawImage(CGImageRef image, CGRect src, CGRect dst);
    void StrokePath(id path, __CGColorQuad color, float lineWidth, CGAffineTransform* t);
};
